;======================== header ===============================================
;+
; NAME:
;   chipswb
; PURPOSE:
;   make zip files for Solar Weather Browser for MLSO PICS (H-alpha) images.
; CATEGORY:
;   SWB version 1.00
; CALLING SEQUENCE:
;   (meant to be used in batchmode)
; KEYWORD PARAMETERS:
;   date: yyyymmdd
;   /logfile: to print diagnostics to a logfile (default=stdout=screen)
;   /new:	if set, process ONLY new images.
;   /view:	if set, display processed images in an IDL window.
; MODIFICATION HISTORY:
; Created by Andrew L. Stanger, ROB, 04 Jun 2007: Adapted from "mk42zip.pro".
; 14 Jun 2007 [ALS]; Add rotation to correct for solar P-angle.
; 21 Jun 2007 [ALS]: Set in/out directories for HAO/MLSO archive.
; 27 Jun 2007 [ALS]: Smooth image: img = smooth (img, 3) .
; 29 Jun 2007 [ALS]: Set pixels in outer FOV to zero.
; 31 Jul 2007 [ALS]: Add "view" keyword to display image in IDL window.
; 03 Aug 2007 [ALS]: Rename from "chip2zip.pro" to "chip2swb.pro".
; 10 Aug 2007 [ALS]: Remove "hdr" parameter in write2zip.
; 18 Sep 2007 [ALS]: Rename from "chip2swb" to "chipswb".  Add "new" keyword.
;  1 Oct 2007 [ALS]: Use "scale2swb_mlso.pro".
; 22 Oct 2007 [ALS]; Replace "write2zip.pro" with "write2swb.pro". Use z-buffer.
;-
;======================== end header ===========================================

PRO chipswb, date=date, logfile=logfile, new=new, view=view

; ------------------------------------------------------------------------------
; Procedure name.
; ------------------------------------------------------------------------------

myname = 'chipswb'

; ------------------------------------------------------------------------------
; Define SWB reference parameters.
; ------------------------------------------------------------------------------

set_swbs_mlso

outdim        = !swbs.outdim			; 512
ref_solradius = !swbs.ref_solradius		; 184
ref_xcenter   = !swbs.ref_xcenter		; 256
ref_ycenter   = !swbs.ref_ycenter		; 256

;----------------------------------------------------------------------
; Use Z-buffer unless view keyword is set.
; ------------------------------------------------------------------------------

old_device = !D.NAME
IF (NOT KEYWORD_SET (view)) THEN	$
BEGIN
    SET_PLOT, 'Z'
    DEVICE, SET_RESOLUTION = [outdim, outdim]		; output image size.
    DEVICE, z_buffering = 0				; 0 for 2D.
END

; ------------------------------------------------------------------------------
; Get year, month & day from date parameter.
; ------------------------------------------------------------------------------

year  = strmid (date, 0, 4)
month = strmid (date, 4, 2)
day   = strmid (date, 6, 2)

; ------------------------------------------------------------------------------
; Input data directory.
; ------------------------------------------------------------------------------

;datadir = !swbs.data_dir + year + '/' + month + '/' + day + '/'

datadir = '/archive/acos/' + year + '/' + month + '/' + day + '/'

print, 'datadir: ', datadir

; ------------------------------------------------------------------------------
; Open Log file.
; ------------------------------------------------------------------------------

logfilename = !swbs.log_dir + myname + '/' + myname + '.log'

IF (keyword_set (logfile)) THEN $
   openw, log, logfilename, /get_lun, /append, bufsize=0 $
ELSE log = -1

; ------------------------------------------------------------------------------
; First log message.
; ------------------------------------------------------------------------------

printf, log, ' >>>> ' + myname + '.pro started at ' + systime ()

; ------------------------------------------------------------------------------
; If "view" keyword is set, open IDL window.
; ------------------------------------------------------------------------------

IF (keyword_set (view))  THEN	$
    window, 4, xs=outdim, ys=outdim, retain=2

; ------------------------------------------------------------------------------
; Create list of images to process.
; ------------------------------------------------------------------------------

sdir = datadir + '*chp.hsh.fts*'
print, 'sdir: ', sdir
;spawn, 'ls ' + sdir, list

list = file_search (datadir + '*chp.hsh.fts*')

; ------------------------------------------------------------------------------
; Any FITS images to be processed ?
; ------------------------------------------------------------------------------

IF (list [0] EQ '') THEN $
BEGIN ;{ ---------- Empty List ----------
    print,       ' No images available for: ', datadir
    printf, log, ' No images available for: ', datadir
END $

ELSE $
BEGIN ;{ ------- List not empty -------------

    IF (KEYWORD_SET (new))  THEN	$
    BEGIN ;{
	; ----------------------------------------------------------------------
	; Create a list of times of FITS images.
	; ----------------------------------------------------------------------
        plist = swb_readlist (date, stlf='chip', odir='/archive/swbs/mlso/')
	plist = STRMID (plist, STRPOS (plist [0], '.jpg') - 15, 15)

	; ----------------------------------------------------------------------
	; Create a list of times of previously processed images.
	; ----------------------------------------------------------------------
	dlist = STRMID (list,  STRPOS ( list [0], '/' + date + '.') + 1, 8)
	tlist = STRMID (list,  STRPOS ( list [0], '/' + date + '.') + 10, 6)
	ilist = dlist + '_' + tlist

	; ----------------------------------------------------------------------
	; Compare the two lists.
	; ----------------------------------------------------------------------

	nlist = cmset_op (ilist, 'AND', /not2, plist, /index)

;	print, 'dlist: '
;	print,  dlist
;	print, 'tlist: '
;	print,  tlist
;	print, 'size (ilist): ', size (ilist)

	print, 'ilist: '
	print,  ilist

	print, 'plist: '
	print,  plist

	print, 'nlist: '
	print,  nlist

	; ----------------------------------------------------------------------
	; Create a list of images to be processed.
	; ----------------------------------------------------------------------

	if (nlist[0] NE -1) THEN	$
	    list = list [nlist]		$
	ELSE				$
	    BEGIN
		print, 'No new images to process.'
	        goto, end_of_process
	    END				
    END 

    ; --------------------------------------------------------------------------
    ; Image processing loop.
    ; --------------------------------------------------------------------------

    FOR i= 0, n_elements (list) - 1 DO	$
    BEGIN ;{

    fitsfile = list [i]
;    print, 'fitsfile: ', fitsfile

    ; --------------------------------------------------------------------------
    ; Report image being processed.
    ; --------------------------------------------------------------------------

    printf, log, ''
    printf, log, 'input  image ', list[i]
    print,       'input  image ', list[i]

    ; --------------------------------------------------------------------------
    ; Read FITS image and header.
    ; --------------------------------------------------------------------------

    img = mlso_readfits (list [i], index, /silent)

    IF (size (img, /n_dim) EQ 0) THEN	$
    BEGIN
        printf, log, "chip2swb: Skipping file " + readdir + list [i]
        continue
    ENDIF

    ; --------------------------------------------------------------------------
    ; Date and Time.
    ; --------------------------------------------------------------------------

    chip_date = index.date_obs
    chip_time = index.time_obs
    date     = index.date_obs + "T" + index.time_obs

    year     = strmid (chip_date,  0, 4)
    month    = strmid (chip_date,  5, 2)
    day      = strmid (chip_date,  8, 2)

    hours    = strmid (chip_time,  0, 2)
    mins     = strmid (chip_time,  3, 2)
    sec      = strmid (chip_time,  6, 2)

    ; --------------------------------------------------------------------------
    ; Rotation angle & sun center.
    ; --------------------------------------------------------------------------

    rot_ang = -index.crota1			; rotation angle.
    xcen    = index.crpix1			; x-axis center.
    ycen    = index.crpix2			; y-axis center.

    print, 'xcen, ycen : ', xcen, ycen

    ; --------------------------------------------------------------------------
    ; Rotate image to correct for the P-angle.
    ; --------------------------------------------------------------------------

    IF (index.crota1 NE 0.0) THEN	$
    BEGIN
        img = rot (img, rot_ang, 1.0, xcen, ycen,  missing=0, /interp)
        print, 'Rotate image.  rot_ang: ', rot_ang
    END

    ; --------------------------------------------------------------------------
    ; Crop image to a square [1024x1024].
    ; --------------------------------------------------------------------------

    xdim   = index.naxis1
    ydim   = index.naxis2

    x1 = (xdim - 1024) / 2
    x2 = x1 + 1023
    y1 = (ydim - 1024) / 2
    y2 = y1 + 1023

    IF (xdim GE 1024 AND ydim GE 1024) THEN	$
    BEGIN
        imgsq = img [x1:x2, y1:y2] 			; Crop image: 1024x1024
    END					$
    ELSE					$
    BEGIN
        print, 'invalid image size: ', xdim, ydim
        RETURN
    END

;    print, 'xdim, ydim: ', xdim, ydim
;    print, 'x1, x2, y1, y2: ', x1, x2, y1, y2

    ; --------------------------------------------------------------------------
    ; Square image parameters.
    ; --------------------------------------------------------------------------

    size_img = size (imgsq)			; size of croped image.
    sxdim = size_img [1]
    sydim = size_img [2]

    sxcen  = (index.crpix1 - x1)		; x-axis sun center: square.
    sycen  = (index.crpix2 - y1)		; y-axis sun center: square.

;    print, 'sxdim, sydim: ', sxdim, sydim

    ; --------------------------------------------------------------------------
    ; Solar radius.
    ; --------------------------------------------------------------------------

    pixrs   = index.crradius			; pixels / Rsun

;    print, 'pixrs   : ', pixrs

    ; --------------------------------------------------------------------------
    ; Image brightness min/max values.
    ; --------------------------------------------------------------------------

    dispmin = index.dispmin		; display minimum.
    dispmax = index.dispmax		; display maximum.

;    dmin = -1200			; experimentally-determined scaling.

    dmin = - 600
    dmax =    50

    print, 'dispmin, dispmax: ', dispmin, dispmax
    print, 'dmin, dmax      : ', dmin, dmax

    ; --------------------------------------------------------------------------
    ; Set pixels in outer FOV to zero.
    ; --------------------------------------------------------------------------

    print, 'Mask off outer FOV pixels.'
    icolor = dmin - 1

    imgsq = disc_mask (imgsq, icolor, sxdim, sydim, sxcen, sycen, pixrs) 

    ; --------------------------------------------------------------------------
    ; Re-size image to SWB reference resolution.
    ; --------------------------------------------------------------------------

    print, 'Resize image.'

    imgswb = scale2swb_mlso (xcen, ycen, pixrs, imgsq)

    ; --------------------------------------------------------------------------
    ; Adjust pixel brightness, using dispmin & dispmax values.
    ; --------------------------------------------------------------------------

    print, 'Rescale pixel intensities.'

;    imgswb   = imgswb > 0

    imin = min (imgswb, max=imax)
    print, 'imin, imax: ', imin, imax , ' [before intensity scaling]'

    imgswb = bytscl (imgswb, min=dmin, max=dmax)	; Re-scale brightness.

;    imin = min (imgswb, max=imax)
;    print, 'imin, imax: ', imin, imax, ' [after  intensity scaling]'

 ;   mask_im = where (imgswb LE min (imgswb))		; Clip dim values.
 ;   imgswb [mask_im] = 0  

    ; --------------------------------------------------------------------------
    ; Smooth image.
    ; --------------------------------------------------------------------------

    imgswb = smooth (imgswb, 3)

    print, 'Smooth image...width: 3 '

    ; --------------------------------------------------------------------------
    ; Display image (if keyword "view" is set).
    ; --------------------------------------------------------------------------

    IF (keyword_set (view))  THEN	$
        tv, imgswb

    ; --------------------------------------------------------------------------
    ; SWB reference Field-Of-View (FOV).
    ; CHIP original Field-Of-View.
    ; Compute the ratio of CHIP FOV to the SWB reference FOV.
    ; --------------------------------------------------------------------------

    sol_rad   = index.rsun			; arcsec / Rsun
    swb_fov   = sol_rad * outdim / ref_solradius  
    chip_fov  = sxdim * index.cdelt1 
    fov_ratio = chip_fov / swb_fov

    fov_ratio = 1.0			; 1:1 scale, due to scale2swb.

    print, 'sol_rad : ', sol_rad, ' arcsec'

    ; --------------------------------------------------------------------------
    ; Create outdir, outfn, color_table_code.
    ; --------------------------------------------------------------------------

    outfn = 'mlso_chip_' + year + month + day + '_' + hours + mins + sec
    outdir = !swbs.out_dir + year + '/' + month + '/' + day + '/'

;    outdir = '/archive/swbs/mlso/' + year + '/' + month + '/' + day + '/'

    hdr              = ''
    data_type        = 'I'
    data_name        = 'MLSO Helium-I'
    color_table_code = '304' 

    ; --------------------------------------------------------------------------
    ; Create zip file.
    ; --------------------------------------------------------------------------

    print, 'Create jpeg image.'

    write2swb, outfn, outdir, imgswb, data_type, data_name, $
               color_table_code, date, /logfile

    printf, log, 'output image ', outdir + outfn
    print,       'output image ', outdir + outfn
    print,       ''


    ENDFOR ;} i
    ; --------------------------------------------------------------------------
ENDELSE	;} ------- list not empty -------------

; ------------------------------------------------------------------------------
; Last log message.
; ------------------------------------------------------------------------------

spawn, 'date', fulldate   
printf, log, '<<<< ' + myname + ' finished at ' + fulldate 

; ------------------------------------------------------------------------------
; Close log file, if needed.
; ------------------------------------------------------------------------------

end_of_process:
IF (keyword_set (logfile)) THEN free_lun, log   
SET_PLOT, old_device
END
