;======================== header ===============================================
;+
; NAME:
;   find_center
; PURPOSE:
;   make zip files for Solar Weather Browser
; CATEGORY:
;   SWB version 1.00
; CALLING SEQUENCE:
;   (meant to be used in batchmode)
; INPUTS:
;   input_image:  the input image
;
; KEYWORD PARAMETERS:
;   threshold: defines the threshold for the limb-point detection
;               (unused at the moment)
; OUTPUTS:
;   xc, yc:  the center of the solar disc in pixel coordinates
; COMMON BLOCKS:
;   none
; SIDE EFFECTS:
;   unknown
; RESTRICTIONS:
;   unknown
; PROCEDURE:
;
; CALLS:
;
; TO DO:
;   implement the threshold keyword
; MODIFICATION HISTORY:
;   2004/08/31, ver 1.00, D. Berghmans, based on uset2web by F. Clette
;   2004/09/02, DB, now correct handling of non-square images
;   2005/12/13, Thanassis Katsiyannis, bug fixes
;-
;======================== end header ====================================
PRO find_center, input_image, xc, yc, Rsun, threshold=threshold

 ; ------------------------ preamble ------------------------------------
 ; Who am I?
   myname   ='find_center'

   threshold_factor=0.37 ;  for finding limb points
     
     
        image=input_image
	sz=size(image)
	 
	; Target image size
	ix=sz[1] 
	iy=sz[2] 
	imcenter=(ix-1.)/2.
	xcoord=fltarr(ix,iy)
	ycoord=fltarr(ix,iy)
        image=congrid(image,ix,iy, cubic=-0.5)

	; find appropiate intensity range
	vmin=0.               ; minimal value considered
	vmax=max(image) <20000; maximal value considered
	edge2=5
	image_without_edges=image[edge2:ix-1-edge2,edge2:iy-1-edge2]
	totpix=n_elements(image_without_edges)
	hist=histogram(image_without_edges,max=vmax,min=vmin,nbins=(vmax-vmin)+1)
	hist_dim= size(hist, /n_elements)
	;
	hsum=0.
	i=vmin-1
	repeat begin
	      i=i+1
	      hsum=hsum+hist[i]
	      if (i eq hist_dim-1) then begin
	         xc=-1
	         yc=-1
	         rsun=-1
	         return
	      endif   
	endrep until (hsum ge (totpix*0.01))
	valmin=float(i)
	;
	hsum=0.
	i=(vmax-vmin)+1
	repeat begin
	      i=i-1
	      hsum=hsum+hist[i]
	endrep until (hsum ge (totpix*0.0001))
 	valmax=float(i)
        ;
	image=((image-valmin)*((vmax-vmin)/(valmax-valmin)))>vmin
	
	
	; Extract limb points
	threshold=vmin+(vmax-vmin)*threshold_factor
	maskdisk=replicate(0,ix,iy)
	brightdisk=where(image ge threshold,npts)
	if (npts gt 0) then maskdisk[brightdisk]=1
	;--- Morphological closing operation to remove sunspots ----------
	closewidth=ix/50
	zone=replicate(1,closewidth,closewidth)
	maskdisk=erode(dilate(maskdisk,zone),zone)
	;--- Simple mask by intensity thresholding -----------------------
	edge=3
	zone=replicate(1,edge,edge)
	limbpoints=(maskdisk - erode(maskdisk,zone)) > 0
	;--- Rejects bad limb points (artifacts) -------------------------- 
	mm=closewidth+edge
	limbpoints[0      :mm   ,       0:iy-1]=0
	limbpoints[ix-1-mm:ix-1 ,       0:iy-1]=0
 	limbpoints[0      :ix-1 ,       0:mm  ]=0
	limbpoints[0      :ix-1 , iy-1-mm:iy-1]=0


        ;--- Creation of limb points coordinate vector -------------------

        limblist=where(limbpoints gt 0, nlpoints)
        limbdata=fltarr(nlpoints,2)
         radius  =fltarr(nlpoints)
         angle   =fltarr(nlpoints)
         for i=0,iy-1 do xcoord[*,i]=findgen(ix)
         for i=0,ix-1 do ycoord[i,*]=findgen(iy)
         limbdata[indgen(nlpoints),0]=xcoord[limblist[indgen(nlpoints)]]
         limbdata[indgen(nlpoints),1]=ycoord[limblist[indgen(nlpoints)]]

         ;--- Initial guess: center of mass -------------------------------
         weight=total(image)
         xc=total(image*xcoord)/weight
         yc=total(image*ycoord)/weight

         ;*** MAIN ITERATION: begin **************************************** 
 	 
	 Rsun0=0.
         it=1
         nbiter=100 ; maximum number of iterations
  
         repeat begin
           ;
           radius=sqrt((limbdata[*,0]-xc)^2+(limbdata[*,1]-yc)^2)
           angle=atan(limbdata[*,1]-yc,limbdata[*,0]-xc)
           ;
           Rsun=total(radius)/nlpoints   
           varrad=abs(Rsun-Rsun0)
           ;
           tr=total((radius-Rsun)*cos(angle))/nlpoints*2.
           ti=total((radius-Rsun)*sin(angle))/nlpoints*2.
           ;
           xc=xc+ tr
           yc=yc+ ti
           Rsun0=Rsun
           ;
           it=it+1    ; iteration counter
         endrep until ( (max([tr, ti]) lt 0.01 and varrad lt 0.01) or (it eq nbiter) )

 	 ;*** MAIN ITERATION: end **********************************************

END
