;======================== header ===============================================
;+
; NAME:
;   write2swb
; PURPOSE:
;   make jpeg files for Solar Weather Browser
; CATEGORY:
;   SWB version 1.00
; CALLING SEQUENCE:
;   (meant to be used in batchmode)
; INPUTS:
;   outfn: prefix that will be used in the output filename,
;          typically 10 characters long
;          eg outfn= data+STRMID(lastfile, strpos(lastfile,'.')-10,10) ;
;          data = 'WHL' or 'MAG'
;   outdir: output directory where to write
;          (outdir = basedir+'out/'+year+'/'+month+'/'+day+'/')
;   image: rescaled image, ready for submission to write_jpg     
;   hdr: text (string array) containing the original fits header of the image
;   data_type:  data_type='I' or 'O' (image 'I' or  overlay 'O')
;   data_name: to be used in drop down menu of the SWB,
;              eg data_name='GONG magnetogram'
;   color_table_code: eg '171', '195', '304' or '000' (no scaling or colouring).
;                     For overlays always use '000'
;   date: date of observation, something that can be insterted in anytim2jd
;
; KEYWORD PARAMETERS:
;   /logfile: to print diagnostics to a logfile (default=stdout=screen)
;
; OUTPUTS:
;   
; COMMON BLOCKS:
;   none
; SIDE EFFECTS:
;   unknown
; RESTRICTIONS:
;   unknown
; PROCEDURE:
;
; CALLS:
;
; TO DO:
;
; MODIFICATION HISTORY:
;   2004/06/22, ver 1.00, David Berghmans
;   2004/09/08, DB, added PNG writing for overlays
;   2005/02/08, DB, "close, /all" added
;   2005/03/19, RAMVDL, "close, /all" removed again, because it also closes files in calling program!
;   2005/09/29, Thanassis Katsiyannis, fov_ratio field added to 
;                  list.txt as needed for LASCO images, changes to
;                  the loging system
;   2006/05/23  Ingo Baumann: re, gr, bl keywords added to display
;               colored fieldlines from PFSS2zip
;   2006/06/08  svg keyword added
;   2006/06/10  C. Marque: set tmpdir as default directory for creation
;               of temporary files
;   2006/07/10  C. Marque: change svg keyword meaning: now just look for
;               a svg file 
;   2007/01/15  C. Marque: Removes progressive keyword in write_jpeg
;   2007/08/07, C. Marque: Change the way jpeg files are created.
;               Don't produce png overlays anymore, 
;               no zip files if only jpeg.
;               Don't put crappy files on the server. Change call to write2zip.
; 2007/10/22 Andrew L. Stanger: call set_swbs_mlso.   
;-
;======================== end header ===========================================
PRO write2swb, outfn, outdir, image, data_type, data_name, $
    color_table_code, date, fov_ratio=fov_ratio, logfile=logfile, $
    verbose=verbose, txtfile=txtfile
;
; ------------------------ preamble ------------------------------------
; Who am I?

myname   ='write2zip'

; ------------------------------------------------------------------------------
; Set generic parameters
; ------------------------------------------------------------------------------

set_swbs_mlso

; ------------------------------------------------------------------------------
; log file control
; ------------------------------------------------------------------------------

logfilename=!swbs.log_dir+myname+'/'+myname+'.log'
if keyword_set(logfile) then $
   openw, wzlog, logfilename, /get_lun, /append, bufsize=0 $
else wzlog=-1

; ------------------------------------------------------------------------------
; if /logfile keyword is set, open the logfile, if not
; send output to stout (typically the screen).
; say hello
; ------------------------------------------------------------------------------

if keyword_set(verbose) then $
   printf, wzlog, '>>>> '+myname+'.pro started at '+ systime()

; ------------------------main code ------------------------------------
; change to outdir, if it does not exist, create it.
; ------------------------------------------------------------------------------

if not file_test(outdir) then begin
   file_mkdir, outdir
   printf, wzlog, 'Write2zip:  Created outdir: ', outdir
endif

; ------------------------------------------------------------------------------
; if additionnal text file is included
; ------------------------------------------------------------------------------

if keyword_set(txtfile) then begin
   openw, numfile, !swbs.tmp_dir+outfn+'.txt',/get_lun
   nlines=n_elements(txtfile)
   for line_i=0, nlines-1 do printf, numfile, hdr[line_i]
   close, numfile & free_lun, numfile
endif

; ------------------------------------------------------------------------------
; write the image
; ------------------------------------------------------------------------------

if (data_type eq 'I' or data_type eq 'C') then begin
   file_ext='.jpg'
   swb_write_jpg, outfn, image, quality=90
;prepare output
   if keyword_set(txtfile) then begin
      spawn,'zip -j -qq '+outdir+outfn+'.zip '+$
      !swbs.tmp_dir+outfn+file_ext+' '+!swbs.tmp_dir+outfn+'.txt' 
;clean up tmp directory
      spawn,'rm '+!swbs.tmp_dir+outfn+file_ext+' '+!swbs.tmp_dir+outfn+'.txt'
      file_ext='.zip' 
   endif else begin
       spawn,'du '+!swbs.tmp_dir+outfn+file_ext+' | awk -F '' '' ''{print $1}'' ',ofsize 
       if float(ofsize) lt 10 then begin
       print,!swbs.tmp_dir+outfn+file_ext 
       spawn,'rm '+!swbs.tmp_dir+outfn+file_ext
       goto,bye
       endif else spawn,'mv '+!swbs.tmp_dir+outfn+file_ext+' '+outdir
   endelse
endif

; ------------------------------------------------------------------------------
; write overlays in svg
; ------------------------------------------------------------------------------

if (data_type eq 'O') then begin
   file_ext='.svg'
   if keyword_set(txtfile) then begin
      spawn,'zip -j -qq '+outdir+outfn+'.zip '+$
      !swbs.tmp_dir+outfn+file_ext+' '+!swbs.tmp_dir+outfn+'.txt'
; ------------------------------------------------------------------------------
;clean up tmp directory. note: -j in zip command prevents the path to be stored
; into the zip file
; ------------------------------------------------------------------------------
      spawn,'rm '+!swbs.tmp_dir+outfn+file_ext+' '+!swbs.tmp_dir+outfn+'.txt'
      file_ext='.zip'
   endif else begin
      spawn,'zip -j -qq '+outdir+outfn+'.zip '+$
      !swbs.tmp_dir+outfn+file_ext

; ------------------------------------------------------------------------------
;clean up tmp directory
; ------------------------------------------------------------------------------

      spawn,'rm '+!swbs.tmp_dir+outfn+file_ext
      file_ext='.zip'
   endelse
endif

; ------------------------------------------------------------------------------
; tell the logfile         
; ------------------------------------------------------------------------------

if keyword_set(verbose) then $
   printf, wzlog, 'Write2zip: Zip file '+outdir+outfn+'.zip was created'

; ------------------------------------------------------------------------------
; update list
; ------------------------------------------------------------------------------

cd,outdir;;; in that case it's easier to work directly in outdir
listfn='list.txt'

; ------------------------------------------------------------------------------
; calculate secs since Jan, 1, 1970
; ------------------------------------------------------------------------------

jd  = anytim2jd(date)  
ref = anytim2jd('1970-01-01T00:00:00.000Z')
jd.int=jd.int-ref.int
jd.frac=jd.frac-ref.frac
sec= jd.int*24*60*60  + LONG(ROUND(jd.frac*24*60*60))

line=data_type+'|'+data_name
if keyword_set(fov_ratio) then begin
   fov_ratio_strg = string(fov_ratio, format='(f6.3)')
   fov_ratio_strg = strtrim(fov_ratio_strg,2)
   line=line+'|'+date+'|'+STRTRIM(sec,2) +'|'+outfn+file_ext+$
   '|'+color_table_code+'|'+fov_ratio_strg
endif else $
   line=line+'|'+date+'|'+STRTRIM(sec,2) +'|'+outfn+file_ext+$
   '|'+color_table_code
line_exists=0
if file_exist(listfn) then begin

   openr, listfnnr, listfn, /get_lun
   readline=''
   inputlistfn=''
   while not EOF(listfnnr) do begin 
	 readf, listfnnr, readline
	 inputlistfn=[inputlistfn, readline]
   endwhile
   close, listfnnr & free_lun, listfnnr

; ------------------------------------------------------------------------------
; check if current line exists already
; ------------------------------------------------------------------------------

   line_exists=max(line EQ inputlistfn[*])
   if not line_exists then inputlistfn=[inputlistfn, line]
endif else inputlistfn=['',line]

if not line_exists then begin
   inputlistfn=inputlistfn[1:*]
   openw, listfnnr, listfn, /get_lun
   for i= 0, n_elements(inputlistfn)-1 DO $
   printf, listfnnr, inputlistfn[i]
   close, listfnnr & free_lun, listfnnr
endif

; ------------------------------------------------------------------------------
; sort list and remove double entries
;    spawn,'sort list.txt > list.txt2'
; ------------------------------------------------------------------------------

spawn,'sort list.txt |uniq --check-chars=81 > list.txt2'
spawn,'mv list.txt2 list.txt'

; ------------------------------------------------------------------------------
; say bye
; ------------------------------------------------------------------------------

bye:
if keyword_set(verbose) then $
   printf, wzlog, '<<<< '+myname+'.pro finished at '+ systime()   
if keyword_set(logfile) then begin
   close, wzlog
   free_lun, wzlog
endif
end
