;===============================================================================
; mk4swb.pro	IDL procedure
;===============================================================================
;+
; NAME:
;   mk4swb
; PURPOSE:
;   make JPEG files for Solar Weather Browser for MLSO Mark-IV images.
; CATEGORY:
;   SWB version 1.00
; CALLING SEQUENCE:
;   mk4swb, date='20070721', /logfile, /new
;   (meant to be used in batchmode)
; KEYWORD PARAMETERS:
;   date	 yyyymmdd
;   /logfile	print diagnostics to a logfile (default=stdout=screen)
;   /new	if set, process only new images (not previously processed).
;   /view	if set, display processed image in IDL window.
; MODIFICATION HISTORY:
; 'cor1zip.pro' sreated C.Marque, ROB, Feb 21, 2007.
; Inspired by original LASCO2zip
; 2007/05/14 [CM]: clean up code, make custom output directory for SECCHI 
;                     SWB archives at ROB                          
;
; Andrew L. Stanger [ALS] 29 May 2007: 'mk42zip.pro' adapted from "cor2zip.pro".
; 14 Jun 2007 [ALS]: Display Mk4 image in IDL window.
; 20 Jun 2007 [ALS]: Adapt to HAO computing environment.
; 17 Jun 2007 [ALS]: Set occulting disc & outer FOV pixels to zero.
; 31 Jul 2007 [ALS]: Add "view" keyword to display image in an IDL window.
; 03 Aug 2007 [ALS]: Rename from "mk42zip" to "mk42swb".
; 17 Sep 2007 [ALS]: Rename from "mk42swb" to "mk4swb".  Add "/new" keyword.
; 22 Oct 2007 [ALS]; Replace "write2zip.pro" with "write2swb.pro". Use Z-buffer.
;-
;===============================================================================

PRO mk4swb, date=date, logfile=logfile, new=new, view=view

; ------------------------------------------------------------------------------
; Procedure name.
; ------------------------------------------------------------------------------

myname = 'mk4swb'

orig_dir = curdir ()				; save current directory.

; ------------------------------------------------------------------------------
; Define SWB reference parameters.
; ------------------------------------------------------------------------------

set_swbs_mlso

outdim        = !swbs.outdim			; 512
ref_solradius = !swbs.ref_solradius		; 184
ref_xcenter   = !swbs.ref_xcenter		; 256
ref_ycenter   = !swbs.ref_ycenter		; 256

print, '!swbs: ', !swbs

; ------------------------------------------------------------------------------
; Use Z-buffer unless view keyword is set.
; ------------------------------------------------------------------------------

old_device = !D.NAME
IF (NOT KEYWORD_SET (view)) THEN	$
BEGIN
    SET_PLOT, 'Z'
    DEVICE, SET_RESOLUTION = [outdim, outdim]		; output image size.
    DEVICE, z_buffering = 0				; 0 for 2D.
END

; ------------------------------------------------------------------------------
; Get year, month & day from date parameter.
; ------------------------------------------------------------------------------

IF (NOT KEYWORD_SET (date)) THEN	$
BEGIN
    print, "date parameter (e,g, date='20070721') is required."
    goto, end_of_process
END

year  = strmid (date, 0, 4)
month = strmid (date, 4, 2)
day   = strmid (date, 6, 2)

; ------------------------------------------------------------------------------
; Input data directory.
; ------------------------------------------------------------------------------

;datadir = '/archive/acos/' + year + '/' + month + '/' + day + '/'

datadir = !swbs.data_dir + year + '/' + month + '/' + day + '/'

print, 'datadir: ', datadir

; ------------------------------------------------------------------------------
; Output data directory.
; ------------------------------------------------------------------------------

;outdir = '/archive/swbs/mlso/' + year + '/' + month + '/' + day + '/'

outdir = !swbs.out_dir + year + '/' + month + '/' + day + '/'

print, 'outdir: ', outdir

; ------------------------------------------------------------------------------
; Open Log file.  Write first log message.
; ------------------------------------------------------------------------------

IF (KEYWORD_SET (logfile)) THEN $
BEGIN
    logfilename = !swbs.log_dir + myname + '/' + myname + '.log'
    openw, log, logfilename, /get_lun, /append, bufsize=0
    print, 'logfile: ', logfilename
END	$
ELSE log = -1

printf, log, ' >>>> ' + myname + '.pro started at ' + systime ()

; ------------------------------------------------------------------------------
; Open IDL window (if "view" keyword is set).
; ------------------------------------------------------------------------------

IF (KEYWORD_SET (view))  THEN	$
    window, 0, xs=outdim, ys=outdim, retain=2

; ------------------------------------------------------------------------------
; Create a list of FITS images for the specified day.
; ------------------------------------------------------------------------------

sdir = datadir + '*mk4.rpb.vig.fts*'
print, 'sdir: ', sdir

;spawn, 'ls ' + sdir, list

list = file_search (sdir)

;size_list = size (list)
;print, 'size (list): ', size_list
;print, 'size_list [0]]', size_list [0]

print, 'list: '
print, list

; ------------------------------------------------------------------------------
; Continue only if FITS list is not empty.
; ------------------------------------------------------------------------------

IF (list [0] EQ '') THEN print, '*** No FITS images available for: ', datadir

IF (list [0] EQ '') THEN $
BEGIN
    printf, log, '*** No FITS images available for: ', datadir
    goto, end_of_process
END $

ELSE $
BEGIN ;{ ------- list not empty -------------

    IF (KEYWORD_SET (new))  THEN	$
    BEGIN ;{
	; ----------------------------------------------------------------------
	; Create a list of times of FITS images for the specified day.
	; ----------------------------------------------------------------------

        plist = swb_readlist (date, stlf = 'mk4', odir = '/archive/swbs/mlso/')
	plist = STRMID (plist, STRPOS (plist [0], '.jpg') - 15, 15)

	; ----------------------------------------------------------------------
	; Create a list of JPEG (previously-processed) images (if any exist).
	; ----------------------------------------------------------------------

	dlist = STRMID (list,  STRPOS ( list [0], '/' + date + '.') +  1, 8)
	tlist = STRMID (list,  STRPOS ( list [0], '/' + date + '.') + 10, 6)
	ilist = dlist + '_' + tlist

	; ----------------------------------------------------------------------
	; Compare the FITS list with the JPEG list.
	; ----------------------------------------------------------------------

	nlist = cmset_op (ilist, 'AND', /not2, plist, /index)

;	print, 'dlist: '
;	print,  dlist
;	print, 'tlist: '
;	print,  tlist
;	print, 'size (ilist): ', size (ilist)

	print, 'ilist: '
	print,  ilist

	print, 'plist: '
	print,  plist

	print, 'nlist: '
	print,  nlist

	; ----------------------------------------------------------------------
	; Create a list of images to be processed.
	; ----------------------------------------------------------------------

	IF (nlist [0] NE -1) THEN	$
	    list = list [nlist]		$
	ELSE				$
	    BEGIN
		print, 'No new images to process.'
	        goto, end_of_process
	    END				
    END  ;}

    ; --------------------------------------------------------------------------
    ; Image processing loop.
    ; --------------------------------------------------------------------------

    FOR i= 0, n_elements (list) - 1 DO	$
    BEGIN ;{

    fitsfile = list [i]
;    print, 'fitsfile: ', fitsfile

    ; --------------------------------------------------------------------------
    ; Report start of image processing.
    ; --------------------------------------------------------------------------

    printf, log, ''
    printf, log, 'Input image ', list[i]
    print,       'Input image ', list[i]

    ; --------------------------------------------------------------------------
    ; Read image and header.
    ; --------------------------------------------------------------------------

    img = mlso_readfits (list [i], index, /silent)

    IF (size (img, /n_dim) EQ 0) THEN	$
    BEGIN
        printf, log, "Skipping file " + readdir + list [i]
        continue
    ENDIF

    ; --------------------------------------------------------------------------
    ; Date and Time.
    ; --------------------------------------------------------------------------

    mk4_date = index.date_obs
    mk4_time = index.time_obs
    date     = index.date_obs + "T" + index.time_obs

    year     = strmid (mk4_date,  0, 4)
    month    = strmid (mk4_date,  5, 2)
    day      = strmid (mk4_date,  8, 2)

    hours    = strmid (mk4_time,  0, 2)
    mins     = strmid (mk4_time,  3, 2)
    sec      = strmid (mk4_time,  6, 2)

    ; --------------------------------------------------------------------------
    ; Solar radius in arcsec.
    ; --------------------------------------------------------------------------

    sol_rad = index.rsun

    ; --------------------------------------------------------------------------
    ; Image brightness min/max values.
    ; --------------------------------------------------------------------------

    datamin = FIX (index.datamin)		; data    minimum.
    datamax = FIX (index.datamax)		; data    maximum.
    dispmin = FIX (index.dispmin)		; display minimum.
    dispmax = FIX (index.dispmax)		; display maximum.

    imin = min (img, max=imax)

    print, 'datamin/datamax: ', datamin, datamax
    print, 'dispmin/dispmax: ', dispmin, dispmax
    print, 'imin/imax:       ', imin, imax

    ; --------------------------------------------------------------------------
    ; Re-size image.
    ; --------------------------------------------------------------------------

    icolor =   0
    mag = float (outdim) / float (index.naxis1)	; Magnification factor.
    oxcen = index.crpix1 * mag				; x-axis sun center.
    oycen = index.crpix2 * mag				; y-axis sun center.
    opixrs = index.crradius * mag			; pixels/Rsun

    print, 'naxis1:   ', index.naxis1
    print, 'mag:      ', mag
    print, 'crradius: ', index.crradius
    print, 'opixrs:   ', opixrs

    img = img > 0
    img = congrid (img, outdim, outdim)		; Re-size imimage.

    ; --------------------------------------------------------------------------
    ; Set pixels inside occulting disk and outer FOV to zero.
    ; --------------------------------------------------------------------------

    radmin = 1.15
    radmax = 2.80
    img = corona_mask (img, icolor, outdim, outdim, oxcen, oycen, opixrs, $
                       rmin=radmin, rmax=radmax)

    printf, log, 'corona mask rmin/rmax: ', radmin, radmax

    ; --------------------------------------------------------------------------
    ; Smooth image.
    ; --------------------------------------------------------------------------

    img = smooth (img, 3)

    printf, log, 'smooth image factor: 3'
    ; --------------------------------------------------------------------------
    ; Adjust pixel brightness, using dispmin & dispmax values.
    ; --------------------------------------------------------------------------

    imin = min (img, max=imax)
    print, '  imin/imax:       ', imin, imax

;    img   = bytscl (img, dispmin, dispmax)		; Re-scale brightness.

    ; For now, use a high-contrast linear scaling of brightness.
 
;    dmin = - 50
;    dmax =  300

    dmin =  -50
    dmax =  400

;    dmax = datamax * 0.60
;    dmax = imax

    printf, log, 'dmin/dmax: ', dmin, dmax

    img = bytscl (img, min=dmin, max=dmax, top=250)	; Re-scale brightness.

;    img = hist_equal (img)			; Histogram equalization.

    mask_im = where (img LE min (img))		; Clip dim values.
    img [mask_im] = 0  

    print, 'Adjust intensity scaling ... dmin/dmax: ', dmin, dmax
    imin = min (img, max=imax)
    print, '  imin/imax:       ', imin, imax

    ; --------------------------------------------------------------------------
    ; Display image (if "view" keyword is set).
    ; --------------------------------------------------------------------------

    IF (KEYWORD_SET (view))  THEN	$
        tv, img

    ; --------------------------------------------------------------------------
    ; SWB's standard Field Of View (FOV).
    ; --------------------------------------------------------------------------

    swb_fov = sol_rad * outdim / ref_solradius  

    ; --------------------------------------------------------------------------
    ; Calculate the ratio of COR's FOV to the standard SWB's FOV.
    ; --------------------------------------------------------------------------

    mk4_fov = index.naxis1 * index.cdelt1 
    fov_ratio = mk4_fov / swb_fov

    ; --------------------------------------------------------------------------
    ; Create outfn, color_table_code.
    ; --------------------------------------------------------------------------

    outfn  = 'mlso_mk4_' + year + month + day + '_' + hours + mins + sec

    hdr              = ''
    data_type        = 'C'
    data_name        = 'MLSO Mk4'
    color_table_code = '001' 

    ; --------------------------------------------------------------------------
    ; Create zip file.
    ; --------------------------------------------------------------------------

    write2swb, outfn, outdir, img, data_type, data_name, $
               color_table_code, date, fov_ratio=fov_ratio, /logfile

    printf, log, 'output image ', outdir + outfn
    print,       'output image ', outdir + outfn
    print,       ''

    ; cd, !swbs.source_dir + 'bin/' + myname

    ENDFOR ;} end of image processing loop.
    ; --------------------------------------------------------------------------

ENDELSE	;} ------- list not empty -------------

end_of_process:

cd, orig_dir

; ------------------------------------------------------------------------------
; Last log message.
; Close log file, if needed.
; ------------------------------------------------------------------------------

spawn, 'date', fulldate   
printf, log, '<<<< ' + myname + ' finished at ' + fulldate 

IF (KEYWORD_SET (logfile)) THEN free_lun, log   
SET_PLOT, old_device

END
