;======================== header ===============================================
;+
; NAME:
;   picsd2swb
; PURPOSE:
;   Make jpg files for Solar Weather Browser:
;   MLSO PICS (H-alpha Disc "Raw") images.
; CATEGORY:
;   SWB version 1.00
; CALLING SEQUENCE:
;   (meant to be used in batchmode)
; KEYWORD PARAMETERS:
;   /logfile: to print diagnostics to a logfile (default=stdout=screen)
;   date: yyyymmdd
; MODIFICATION HISTORY:
; Created by Andrew L. Stanger, ROB, 04 Jun 2007: Adapted from "mk42zip.pro".
; 06 Jun 2007 [ALS]: Use square root scaling.
; 07 Jun 2007 [ALS]: Add rotation to correct for solar P-angle.
; 21 Jun 2007 [ALS]: Set in/out directories for HAO/MLSO archive.
; 26 Jul 2007 [ALS]: Add "view" keyword to display image in an IDL window.
; 31 Jul 2007 [ALS]; Use scale2swb to resize image.
; 02 Aug 2007 [ALS]: Intensity scaling: power (2), min=imax*0.15, max=imax.
;		     Rename from "picsd2zip.pro" to "picsd2swb.pro".
;-
;======================== end header ===========================================

PRO picsd2swb, logfile=logfile, date=date, view=view

; ------------------------------------------------------------------------------
; Procedure name.
; ------------------------------------------------------------------------------

myname = 'picsd2zip'

; ------------------------------------------------------------------------------
; Define SWB reference parameters.
; ------------------------------------------------------------------------------

set_swbs

outdim        = !swbs.outdim			; 512
ref_solradius = !swbs.ref_solradius		; 184
ref_xcenter   = !swbs.ref_xcenter		; 256
ref_ycenter   = !swbs.ref_ycenter		; 256

print, 'outdim: ', outdim
print, 'ref_soladius: ', ref_solradius
print, 'ref_xcenter:  ', ref_xcenter
print, 'ref_ycenter:  ', ref_ycenter

; ------------------------------------------------------------------------------
; Get year, month & day from date parameter.
; ------------------------------------------------------------------------------

IF (keyword_set (date))  THEN	$
BEGIN
    year  = strmid (date, 0, 4)
    month = strmid (date, 4, 2)
    day   = strmid (date, 6, 2)
END	$
ELSE	$
BEGIN
   print, "No date specified."
   RETURN
END

; ------------------------------------------------------------------------------
; Input data directory.
; ------------------------------------------------------------------------------

;datadir = !swbs.data_dir + year + '/' + month + '/' + day + '/'
datadir = '/archive/acos/' + year + '/' + month + '/' + day + '/'

print, 'datadir: ', datadir

; ------------------------------------------------------------------------------
; Open Log file.
; ------------------------------------------------------------------------------

logfilename = !swbs.log_dir + myname + '/' + myname + '.log'

IF (keyword_set (logfile)) THEN $
   openw, log, logfilename, /get_lun, /append, bufsize=0 $
ELSE log = -1

; ------------------------------------------------------------------------------
; Say hello.
; ------------------------------------------------------------------------------

printf, log, ' >>>> ' + myname + '.pro started at ' + systime ()

; ------------------------------------------------------------------------------
; If "view" keyword is set, opan an IDL window.
; ------------------------------------------------------------------------------

IF (keyword_set (view))  THEN	$
    window, 1, xs=outdim, ys=outdim, retain=2

; ------------------------------------------------------------------------------
; Create list of images.
; ------------------------------------------------------------------------------

sdir = datadir + '*dpm.asc.fts*'
print, 'sdir: ', sdir

;list = file_search (datadir + '*pics.asc.fts*')

spawn, 'ls ' + sdir, list

; ------------------------------------------------------------------------------
; Image processing loop.
; ------------------------------------------------------------------------------

IF (list [0] EQ ' ') THEN					$
    printf, log, '  No images available for this period'	$
ELSE								$
BEGIN	;{ ------- list not empty -------------

    FOR i= 0, n_elements (list) - 1 DO	$
    BEGIN ;{

    fitsfile = list [i]
    print, 'fitsfile: ', fitsfile

    ; --------------------------------------------------------------------------
    ; Report image being processed.
    ; --------------------------------------------------------------------------

    printf, log, ''
    printf, log, 'Processing image ', list[i]
    printf, log, ''

    ; --------------------------------------------------------------------------
    ; Read image and header.
    ; --------------------------------------------------------------------------

    img = mlso_readfits (list [i], index, /silent)

    IF (size (img, /n_dim) EQ 0) THEN	$
    BEGIN
        printf, log, "picsd2zip: Skipping file " + readdir + list [i]
        continue
    ENDIF

    ; --------------------------------------------------------------------------
    ; Date and Time.
    ; --------------------------------------------------------------------------

    pics_date = index.date_obs
    pics_time = index.time_obs
    date     = index.date_obs + "T" + index.time_obs

    year     = strmid (pics_date,  0, 4)
    month    = strmid (pics_date,  5, 2)
    day      = strmid (pics_date,  8, 2)

    hours    = strmid (pics_time,  0, 2)
    mins     = strmid (pics_time,  3, 2)
    sec      = strmid (pics_time,  6, 2)

    ; --------------------------------------------------------------------------
    ; Solar radius in arcsec.
    ; --------------------------------------------------------------------------

    sol_rad = index.rsun			; arcsec/Rsun

    ; --------------------------------------------------------------------------
    ; Rotation angle & sun center.
    ; --------------------------------------------------------------------------

    rot_ang = -index.crota1			; rotation angle.
    xcen    = index.crpix1			; x-axis center.
    ycen    = index.crpix2			; y-axis center.
    print, 'xcen, ycen : ', xcen, ycen

    ; --------------------------------------------------------------------------
    ; Rotate image to correct for the P-angle.
    ; --------------------------------------------------------------------------

    IF (index.crota1 NE 0.0)  THEN	$
    BEGIN
        img = rot (img, rot_ang, 1.0, xcen, ycen,  missing=0, /interp)
        print, 'Rotate image . . . rot_ang : ', rot_ang
    END

    ; --------------------------------------------------------------------------
    ; Crop image [1024x1024].
    ; --------------------------------------------------------------------------

    xdim = index.naxis1
    ydim = index.naxis2
    x1   = (xdim - 1024) / 2
    x2   = x1 + 1023
    y1   = (ydim - 1024) / 2
    y2   = y1 + 1023

    IF (xdim GE 1024 AND ydim GE 1024) THEN	$
    BEGIN
        imgsq = img [x1:x2, y1:y2] 			; Crop image: 1024x1024
    END					$
    ELSE					$
    BEGIN
        print, 'invalid image size: ', xdim, ydim
        RETURN
    END

    ; --------------------------------------------------------------------------
    ; Square image parameters.
    ; --------------------------------------------------------------------------

    size_img = size (imgsq)
    sxdim    = size_img [1]
    sydim    = size_img [2]

    sxcen    = (index.crpix1 - x1)
    sycen    = (index.crpix2 - y1)
    pixrs    = index.crradius			; pixels/Rsun

    print, 'sxdim, sydim : ', sxdim, sydim

    ; --------------------------------------------------------------------------
    ; Resize image to adhere to SWB reference resolution.
    ; --------------------------------------------------------------------------

    imgswb   = scale2swb (sxcen, sycen, pixrs, imgsq)

    fov_ratio = 1.0

    ; --------------------------------------------------------------------------
    ; Image brightness min/max values.
    ; --------------------------------------------------------------------------

    dispmin = index.dispmin		; display minimum.
    dispmax = index.dispmax		; display maximum.

    ; --------------------------------------------------------------------------
    ; Adjust pixel brightness, using dispmin & dispmax values.
    ; --------------------------------------------------------------------------

    imgswb           = imgswb -  4			; Subtract bkg level.
;    imgswb           = imgswb - 100			; Subtract bkg level.
    mask_im          = where (imgswb LE min (imgswb))	; Clip dim values.
    imgswb [mask_im] = 0  
    imgswb           = imgswb > 0

    imin = min (imgswb, max=imax)
    print, 'imin, imax: ', imin, imax, ' [before brightness scaling]'

;    imgswb = sqrt (imgswb)				; Square root.
    img    = float (imgswb)
    imgswb = img * img
    imin   = min (imgswb, max=imax)			; Get min/max.
    print, 'imin, imax: ', imin, imax, ' [after  power2     scaling]'

;    imin =  12
;    imax =  18

    imgswb = bytscl (imgswb, min=imax*0.15, max=imax)	; Re-scale brightness.

    imin   = min (imgswb, max=imax)			; Get min/max.
    print, 'imin, imax: ', imin, imax, ' [after  brightness scaling]'

    ; --------------------------------------------------------------------------
    ; Sharpen image.
    ; --------------------------------------------------------------------------

;    imgswb = unsharp_mask (imgswb, radius=4, threshold=1)

    ; --------------------------------------------------------------------------
    ; Smooth image.
    ; --------------------------------------------------------------------------

;    imgswb = smooth (imgswb, 3)

;    print, 'Smooth image . . . width: 3'

    ; --------------------------------------------------------------------------
    ; Set pixels in outer FOV to zero.
    ; --------------------------------------------------------------------------

    icolor = 0 
    imgswb = disc_mask (imgswb, icolor, outdim, outdim, $
                        ref_xcenter, ref_ycenter, ref_solradius)

    print, 'Mask off outer FOV [corners].'

    ; --------------------------------------------------------------------------
    ; Display image (if "view" keyword is set).
    ; --------------------------------------------------------------------------

    IF (keyword_set (view))  THEN	$
        tv, imgswb

    ; --------------------------------------------------------------------------
    ; SWB's standard Field Of View (FOV).
    ; --------------------------------------------------------------------------

    swb_fov = sol_rad * outdim / ref_solradius  

    ; --------------------------------------------------------------------------
    ; Calculate the ratio of COR's FOV to the standard SWB's FOV.
    ; --------------------------------------------------------------------------

    pics_fov = index.naxis1 * index.cdelt1 
    fov_ratio = pics_fov / swb_fov

    ; --------------------------------------------------------------------------
    ; Create outdir, outfn, color_table_code.
    ; --------------------------------------------------------------------------

    outfn = 'mlso_picsd_' + year + month + day + '_' + hours + mins + sec

;    outdir = !swbs.out_dir + 'MLSO/' + year + '/' + month + '/' + day + '/'

    outdir = !swbs.out_dir + 'mlso/' + year + '/' + month + '/' + day + '/'

    hdr              = ''
    data_type        = 'I'
    data_name        = 'MLSO H-alpha Disc Raw'
    color_table_code = '001' 

    ; --------------------------------------------------------------------------
    ; Create zip file.
    ; --------------------------------------------------------------------------

    write2zip, outfn, outdir, imgswb, hdr, data_type, data_name, $
               color_table_code, date, /logfile
    ;cd, !swbs.source_dir + 'bin/' + myname
    printf, log, '  processed pics disc image ', list [i]

    ; --------------------------------------------------------------------------

    ENDFOR ;} i
ENDELSE	;} ------- list not empty -------------

; ------------------------------------------------------------------------------
; Last log file entry.
; ------------------------------------------------------------------------------

spawn, 'date', fulldate   
printf, log, ' <<<< ' + myname + ' finished at ' + fulldate 

; ------------------------------------------------------------------------------
; Close log file, if needed.
; ------------------------------------------------------------------------------

IF (keyword_set (logfile)) THEN free_lun, log   
END
