;======================== header ===============================================
;+
; NAME:
;   picslswb
; PURPOSE:
;   Make zip files for Solar Weather Browser:
;   MLSO PICS (H-alpha Limb) images.
; CATEGORY:
;   SWB version 1.00
; CALLING SEQUENCE:
;   (meant to be used in batchmode)
; KEYWORD PARAMETERS:
;   date:	yyyymmdd
;   /logfile:	if set, print diagnostics to a logfile (default=stdout=screen)
;   /new:	if set, process only "new" images.
;   /view:	if set, display processed image in IDL window.
; MODIFICATION HISTORY:
; Created by Andrew L. Stanger, ROB, 04 Jun 2007: Adapted from "mk42zip.pro".
;  6 Jun 2007 [ALS]: Use square root scaling for pixel intensity.
; 13 Jun 2007 [ALS]; Add rotation to correct for solar P-angle.
; 21 Jun 2007 [ALS]: Set in/out directories for HAO/MLSO archive.
; 29 Jun 2007 [ALS]: Set pixels in occulting disc and outer FOV to zero.
; 25 Jul 2007 [ALS]: Add "view" keyword to display image in IDL window.
; 30 Jul 2007 [ALS]: Do "corona_mask" on re-sized image.
; 02 Aug 2007 [ALS]: Rename from "picsl2zip.pro" to picsl2swb.pro"
; 10 Aug 2007 [ALS]: Remove "hdr" parameter from write2zip.
; 18 Sep 2007 [ALS]: Rename from "picsl2swb" to "picslswb".  Add "new" keyword.
;  1 Oct 2007 [ALS]; Corrected output directory name: /archive/swbs/mlso.
; 22 Oct 2007 [ALS]; Replace "write2zip.pro" with "write2swb.pro". Use z-buffer.
;-
;======================== end header ===========================================

PRO picslswb, logfile=logfile, date=date, new=new, view=view

; ------------------------------------------------------------------------------
; Procedure name.
; ------------------------------------------------------------------------------

myname = 'picslswb'

orig_dir = curdir ()				; save current directory.

; ------------------------------------------------------------------------------
; Define SWB reference parameters.
; ------------------------------------------------------------------------------

set_swbs_mlso

outdim        = !swbs.outdim			; 512
ref_solradius = !swbs.ref_solradius		; 184
ref_xcenter   = !swbs.ref_xcenter		; 256
ref_ycenter   = !swbs.ref_ycenter		; 256

print, 'outdim: ', outdim
print, 'ref_solradius: ', ref_solradius
print, 'ref_xcenter:   ', ref_xcenter
print, 'ref_ycenter:   ', ref_ycenter

;----------------------------------------------------------------------
; Use Z-buffer unless view keyword is set.
; ------------------------------------------------------------------------------

old_device = !D.NAME
IF (NOT KEYWORD_SET (view)) THEN	$
BEGIN
    SET_PLOT, 'Z'
    DEVICE, SET_RESOLUTION = [outdim, outdim]		; output image size.
    DEVICE, z_buffering = 0				; 0 for 2D.
END

; ------------------------------------------------------------------------------
; Get year, month & day from date parameter [yyyymmdd].
; ------------------------------------------------------------------------------

IF (NOT KEYWORD_SET (date)) THEN	$
BEGIN
    print, "date parameter (e,g, date='20070721') is required."
    goto, end_of_process
END

year  = strmid (date, 0, 4)
month = strmid (date, 4, 2)
day   = strmid (date, 6, 2)

; ------------------------------------------------------------------------------
; Input data directory.
; ------------------------------------------------------------------------------

datadir = !swbs.data_dir + year + '/' + month + '/' + day + '/'
;datadir = '/archive/acos/' + year + '/' + month + '/' + day + '/'

print, 'datadir: ', datadir

; ------------------------------------------------------------------------------
; Open Log file.
; ------------------------------------------------------------------------------

logfilename = !swbs.log_dir + myname + '/' + myname + '.log'

IF (keyword_set (logfile)) THEN $
   openw, log, logfilename, /get_lun, /append, bufsize=0 $
ELSE log = -1

; ------------------------------------------------------------------------------
; Say hello.
; ------------------------------------------------------------------------------

printf, log, ' >>>> ' + myname + '.pro started at ' + systime ()

; ------------------------------------------------------------------------------
; Open IDL window if "view" keyword is set.
; ------------------------------------------------------------------------------

IF (keyword_set (view))  THEN $
    window, 3, xs=outdim, ys=outdim, retain=2

; ------------------------------------------------------------------------------
; Create list of images.
; ------------------------------------------------------------------------------

sdir = datadir + '*dpm.alr.fts*'
print, 'sdir: ', sdir
;spawn, 'ls ' + sdir, list

list = file_search (sdir)

; ------------------------------------------------------------------------------
; Any images available for processing ?
; ------------------------------------------------------------------------------

IF (list [0] EQ '') THEN $
BEGIN ;{ ---------- Empty List ----------
    print,       ' No images available for: ', datadir
    printf, log, ' No images available for: ', datadir
END $ ;}

ELSE $
BEGIN ;{ ------- List not empty -------------

    IF (KEYWORD_SET (new))  THEN	$
    BEGIN ;{
	; ----------------------------------------------------------------------
	; Create a list of times for images alread processed.
	; ----------------------------------------------------------------------

        plist = swb_readlist (date, stlf='picsl', odir='/archive/swbs/mlso/')
	plist = STRMID (plist, STRPOS (plist [0], '.jpg') - 15, 15)

	; ----------------------------------------------------------------------
	; Create a list of times of FITS image files.
	; ----------------------------------------------------------------------

	dlist = STRMID (list,  STRPOS ( list [0], '/' + date + '.') + 1, 8)
	tlist = STRMID (list,  STRPOS ( list [0], '/' + date + '.') + 10, 6)
	ilist = dlist + '_' + tlist

	; ----------------------------------------------------------------------
	; Compare the two lists.
	; ----------------------------------------------------------------------

	nlist = cmset_op (ilist, 'AND', /not2, plist, /index)

;	print, 'dlist: '
;	print,  dlist
;	print, 'tlist: '
;	print,  tlist
	print, 'size (ilist): ', size (ilist)

	print, 'ilist: '
	print,  ilist

	print, 'plist: '
	print,  plist

	print, 'nlist: '
	print,  nlist

	; ----------------------------------------------------------------------
	; Create a list of FITS images which have not been previously processed.
	; ----------------------------------------------------------------------
	if (nlist[0] NE -1) THEN	$
	    list = list [nlist]		$
	ELSE				$
	    BEGIN
		print,       'No new images to process for: ', datadir
		printf, log, 'No new images to process for: ', datadir
	        goto, end_of_process
	    END				
    END 


    ; --------------------------------------------------------------------------
    ; Image processing loop.
    ; --------------------------------------------------------------------------

    FOR i= 0, n_elements (list) - 1 DO	$
    BEGIN ;{

    fitsfile = list [i]

    ; --------------------------------------------------------------------------
    ; Report image being processed.
    ; --------------------------------------------------------------------------

    printf, log, ''
    printf, log, 'input  image ', list[i]
    print,       'input  image ', list[i]

    ; --------------------------------------------------------------------------
    ; Read image and header.
    ; --------------------------------------------------------------------------

    img = mlso_readfits (list [i], index, /silent)

    IF (size (img, /n_dim) EQ 0) THEN	$
    BEGIN
        printf, log, "picsl2swb: Skipping file " + readdir + list [i]
        continue
    ENDIF

    ; --------------------------------------------------------------------------
    ; Date and Time.
    ; --------------------------------------------------------------------------

    pics_date = index.date_obs
    pics_time = index.time_obs
    date      = index.date_obs + "T" + index.time_obs

    year      = strmid (pics_date,  0, 4)
    month     = strmid (pics_date,  5, 2)
    day       = strmid (pics_date,  8, 2)

    hours     = strmid (pics_time,  0, 2)
    mins      = strmid (pics_time,  3, 2)
    sec       = strmid (pics_time,  6, 2)

    ; --------------------------------------------------------------------------
    ; Rotation angle & sun center.
    ; --------------------------------------------------------------------------

    xdim    = index.naxis1
    ydim    = index.naxis2
    xcen    = index.crpix1			; x-axis center.
    ycen    = index.crpix2			; y-axis center.
    pixrs   = index.crradius			; # pixels / Rsun.
    rot_ang = -index.crota1			; rotation angle.

    print, 'xdim, ydim : ', xdim, ydim
    print, 'xcen, ycen : ', xcen, ycen
    print, 'pixrs      : ', pixrs

    ; --------------------------------------------------------------------------
    ; Rotate image to correct for the P-angle.
    ; --------------------------------------------------------------------------

    IF (index.crota1 NE 0.0)  THEN	$
    BEGIN
       img = rot (img, rot_ang, 1.0, xcen, ycen,  missing=0, /interp)
       print, 'Rotate image . . . rot_ang : ', rot_ang
    END

    ; --------------------------------------------------------------------------
    ; Adjust pixel brightness levels.
    ; --------------------------------------------------------------------------

    dispmin = index.dispmin			; display minimum.
    dispmax = index.dispmax			; display maximum.

;    img   = bytscl (img, dispmin, dispmax)	; Re-scale brightness.

    imin = min (img, max=imax)
    print, 'imin, imax: ', imin, imax, '  [before brightness scaling]'

    img = bytscl (sqrt (img), 2.0, 12.0)	; Square root scaling.

    imin = min (img, max=imax)
    print, 'imin, imax: ', imin, imax, '  [after  brightness scaling]'

    mask_im = where (img LE min (img))		; Clip dim values.
    img [mask_im] = 0  

    ; --------------------------------------------------------------------------
    ; Crop rectangular image to a square [1024x1024].
    ; --------------------------------------------------------------------------

    print, 'Crop image. [1024x1024]'
    xdim = index.naxis1
    ydim = index.naxis2
    x1   = (xdim - 1024) / 2
    x2   = x1 + 1023
    y1   = (ydim - 1024) / 2
    y2   = y1 + 1023

    IF (xdim GE 1024 AND ydim GE 1024) THEN	$
    BEGIN
        imgsq = img [x1:x2, y1:y2] 			; Crop image: 1024x1024
    END						$
    ELSE					$
    BEGIN
        print, 'invalid image size: ', xdim, ydim
        RETURN
    END

    print, 'x1, x2, y1, y2: ', x1, x2, y1, y2
    print, 'size (imgsq): ', size (imgsq)

    ; --------------------------------------------------------------------------
    ; Square image parameters.
    ; --------------------------------------------------------------------------

    size_img = size (imgsq)			; size of croped image.
    sxdim = size_img [1]
    sydim = size_img [2]

    sxcen  = (index.crpix1 - x1)		; x-axis sun center: square.
    sycen  = (index.crpix2 - y1)		; y-axis sun center: square.

    print, 'sxdim, sydim: ', sxdim, sydim

    ; --------------------------------------------------------------------------
    ; Output image parameters.
    ; --------------------------------------------------------------------------

;    mag    = float (outdim) / float (sxdim)	; Magnification factor.
;    oxcen  = sxcen * mag			; x-axis sun center.
;    oycen  = sycen * mag			; y-axis sun center.
;    opixrs = index.crradius * mag		; pixels/Rsun

;    print, 'mag: ', mag
;    print, 'oxcen, oycen: ', oxcen, oycen
;    print, 'opixrs: ', opixrs

    ; --------------------------------------------------------------------------
    ; Shrink image to SWB reference:
    ; size [512x512], pixrs=184, sun center=array center, solar north up.
    ; --------------------------------------------------------------------------

    imgsq   = imgsq > 0

;    imgswb   = congrid (imgsq, outdim, outdim)	; Re-size image.

    imgswb = scale2swb_mlso (sxcen, sycen, pixrs, imgsq) 

    fov_ratio = 1.0

    ; --------------------------------------------------------------------------
    ; Set pixels inside occulting disk and outer FOV to zero.
    ; --------------------------------------------------------------------------

    icolor =   0
    oxdim  = !swbs.outdim
    oydim  = !swbs.outdim
    oxcen  = !swbs.ref_xcenter
    oycen  = !swbs.ref_ycenter
    opixrs = !swbs.ref_solradius

    print, 'oxdim, oydim: ', xdim, ydim
    print, 'size (img): ', size (img)
    print, 'oxcen, oycen: ', oxcen, oycen
    print, 'opixrs: ', opixrs

    img = corona_mask (imgswb, icolor, oxdim, oydim, oxcen, oycen, opixrs, $
                       rmin=0.95)

    ; --------------------------------------------------------------------------
    ; Display image (if "view" keyword is set).
    ; --------------------------------------------------------------------------

    IF (keyword_set (view))  THEN $
        tv, imgswb

    ; --------------------------------------------------------------------------
    ; Solar radius in arcsec.
    ; --------------------------------------------------------------------------

    sol_rad = index.rsun

    ; --------------------------------------------------------------------------
    ; SWB's standard Field Of View (FOV).
    ; --------------------------------------------------------------------------

    swb_fov = sol_rad * outdim / ref_solradius  

    ; --------------------------------------------------------------------------
    ; Calculate the ratio of COR's FOV to the standard SWB's FOV.
    ; --------------------------------------------------------------------------

;    pics_fov = sxdim * index.cdelt1	; arcsec in PICS FOV.
;    fov_ratio = pics_fov / swb_fov

;    print, 'sxdim    " ', sxdim
;    print, 'cdelt1   : ', index.cdelt1
;    print, 'sol_rad  : ', sol_rad
;    print, 'swb_fov  : ', swb_fov
;    print, 'pics_fov : ', pics_fov
    print, 'fov_ratio: ', fov_ratio

    ; --------------------------------------------------------------------------
    ; Create outdir, outfn, color_table_code.
    ; --------------------------------------------------------------------------

    outfn  = 'mlso_picsl_' + year + month + day + '_' + hours + mins + sec
    outdir = !swbs.out_dir + year + '/' + month + '/' + day + '/'

;    outdir = '/archive/swbs/mlso/' + year + '/' + month + '/' + day + '/'

    hdr              = ''
    data_type        = 'C'
    data_name        = 'MLSO H-alpha Limb'
    color_table_code = '001' 			; greyscale.

    ; --------------------------------------------------------------------------
    ; Create zip file.
    ; --------------------------------------------------------------------------

    write2swb, outfn, outdir, imgswb, data_type, data_name, $
               color_table_code, date, /logfile

    ;cd, !swbs.source_dir + 'bin/' + myname
    printf, log, 'output image ', outdir + outfn
    print,       'output image ', outdir + outfn
    print,       ''

    ; --------------------------------------------------------------------------

    ENDFOR ;} i
ENDELSE	;} ------- list not empty -------------

end_of_process:

cd, orig_dir

; ------------------------------------------------------------------------------
; Say bye-bye.
; ------------------------------------------------------------------------------

spawn, 'date', fulldate   
printf, log, '<<<< ' + myname + ' finished at ' + fulldate 

; ------------------------------------------------------------------------------
; Close log file, if needed.
; ------------------------------------------------------------------------------

IF (keyword_set (logfile)) THEN free_lun, log   
SET_PLOT, old_device
END
