;======================== header ===============================================
;+
; NAME:
;   write2zip
; PURPOSE:
;   make zip files for Solar Weather Browser
; CATEGORY:
;   SWB version 1.00
; CALLING SEQUENCE:
;   (meant to be used in batchmode)
; INPUTS:
;   outfn: prefix that will be used in the output filename, typically 10 characters long
;   eg outfn= data+STRMID(lastfile, strpos(lastfile,'.')-10,10) ; data = 'WHL' or 'MAG'
;   outdir: output directory where to write (outdir = basedir+'out/'+year+'/'+month+'/'+day+'/')
;   image: rescaled image, ready for submission to write_jpg     
;   hdr: text (string array) containing the original fits header of the image
;   data_type:  data_type='I' or 'O' (image 'I' or  overlay 'O')
;   data_name: to be used in drop down menu of the SWB, eg data_name='GONG magnetogram'
;   color_table_code: eg '171', '195', '304' or '000' (no scaling or colouring). 
;                     For overlays always use '000'
;   date: date of observation, something that can be insterted in anytim2jd
;
;
; KEYWORD PARAMETERS:
;   /logfile: to print diagnostics to a logfile (default=stdout=screen)

; OUTPUTS:
;   
; COMMON BLOCKS:
;   none
; SIDE EFFECTS:
;   unknown
; RESTRICTIONS:
;   unknown
; PROCEDURE:
;
; CALLS:
;
; TO DO:
;
; MODIFICATION HISTORY:
;   2004/06/22, ver 1.00, David Berghmans
;   2004/09/08, DB, added PNG writing for overlays
;   2005/02/08, DB, "close, /all" added
;   2005/03/19, RAMVDL, "close, /all" removed again, because it also closes files in calling program!
;   2005/09/29, Thanassis Katsiyannis, fov_ratio field added to 
;                  list.txt as needed for LASCO images, changes to
;                  the loging system
;   2006/05/23  Ingo Baumann: re, gr, bl keywords added to display
;               colored fieldlines from PFSS2zip
;   2006/06/08  svg keyword added
;   2006/06/10  C. Marque: set tmpdir as default directory for creation of temporary files
;   2006/07/10  C. Marque: change svg keyword meaning: now just look for a svg file 
;   2007/01/15  C. Marque: Removes progressive keyword in write_jpeg
;-
;======================== end header ===========================================
PRO write2zip, outfn, outdir, image, hdr, data_type, data_name, $
   color_table_code, date, fov_ratio=fov_ratio, logfile=logfile, $
   verbose=verbose, svg=svg ,re, gr, bl

 ; ------------------------ preamble ------------------------------------
 ; Who am I?
   myname   ='write2zip'

 ; main input/output
   ;basedir=getenv('SWBSDIR')
   ;tmpdir=basedir+'SWBS_tmp/'
   set_swbs
 ; log file control
   logfilename=!swbs.log_dir+myname+'/'+myname+'.log'
   IF keyword_set(logfile) THEN $
        openw, wzlog, logfilename, /get_lun, /append, bufsize=0 $
                           ELSE wzlog=-1
    	    ; if /logfile keyword is set, open the logfile, if not
	    ; send output to stout (typically the screen).

 ; say hello
   if keyword_set(verbose) then $
      printf, wzlog, '>>>> '+myname+'.pro started at '+ systime()


 ; ------------------------main code ------------------------------------

    ; change to outdir, if it does not exist, create it.
    IF not file_test(outdir) THEN BEGIN
       file_mkdir, outdir
       printf, wzlog, 'Write2zip:  Created outdir: ', outdir
    ENDIF
;    cd, outdir

    ; write the image
    IF data_type EQ 'I' THEN BEGIN
       file_ext='.jpg'
       write_image, !swbs.tmp_dir+outfn+file_ext,'JPEG', image, quality=90
    ENDIF

    IF data_type EQ 'C' THEN BEGIN
       file_ext='.jpg'
       write_image, !swbs.tmp_dir+outfn+file_ext,'JPEG', image, quality=90
    ENDIF

    IF data_type EQ 'O' THEN BEGIN
        file_ext='.png'
        write_image, !swbs.tmp_dir+outfn+file_ext,'PNG', image
    ENDIF

    ; re, gr, bl necessary for PFSS2zip to display colored fieldlines
    IF KEYWORD_SET (re) THEN BEGIN 
        IF data_type EQ 'O' THEN BEGIN
            file_ext='.png'
            write_image, !swbs.tmp_dir+outfn+file_ext,'PNG', image, re, gr, bl
        ENDIF
    ENDIF


    ; write the hdr
    openw, hdrfile, !swbs.tmp_dir+outfn+'.txt',/get_lun
    nlines=n_elements(hdr)
    for line_i=0, nlines-1 do printf, hdrfile, hdr[line_i]
    close, hdrfile & free_lun, hdrfile
    ;necessary for the spawn command
    cd,!swbs.tmp_dir
    ; zip them together
    IF NOT KEYWORD_SET (svg) THEN spawn,'zip  -qq '+outdir+outfn+'.zip '+outfn+file_ext+' '+outfn+'.txt'
    IF     KEYWORD_SET (svg) THEN BEGIN
;         ;create SVG
;        ;;;keeps smffile temporarily in outdir before the big jump
;        spawn,'~/SWBS/build/svggen/svggen '+outdir+smffilename+' > '+tmpdir+outfn+'.svg' 
;        spawn,'zip  -qq '+outdir+outfn+'.zip '+tmpdir+outfn+file_ext+' '+tmpdir+outfn+'.txt'+' '+tmpdir+outfn+'.svg' 
;        spawn, 'rm '+tmpdir+outfn+'.svg'+' '+outdir+outfn+'.smf'
	spawn,'zip  -qq '+outdir+outfn+'.zip '+outfn+file_ext+' '+outfn+'.txt'+' '+outfn+'.svg'
	spawn, 'rm '+!swbs.tmp_dir+outfn+'.svg'
    ENDIF 
    
    ; remove original files
    spawn, 'rm '+!swbs.tmp_dir+outfn+file_ext+' '+!swbs.tmp_dir+outfn+'.txt'  


    ; tell the logfile         
    if keyword_set(verbose) then $
       printf, wzlog, 'Write2zip: Zip file '+outdir+outfn+'.zip was created'

    ; update list
    cd,outdir;;; in that case it's easier to work directly in outdir
    listfn='list.txt'
    ; calculate secs since Jan, 1, 1970
    jd  = anytim2jd(date)  
    ref = anytim2jd('1970-01-01T00:00:00.000Z')
    jd.int=jd.int-ref.int
    jd.frac=jd.frac-ref.frac
    sec= jd.int*24*60*60  + LONG(ROUND(jd.frac*24*60*60))
    ;
    line=data_type+'|'+data_name

    IF keyword_set(fov_ratio) THEN BEGIN
       fov_ratio_strg = string(fov_ratio, format='(f6.3)')
       fov_ratio_strg = strtrim(fov_ratio_strg,2)
       line=line+'|'+date+'|'+STRTRIM(sec,2) +'|'+outfn+'.zip'+$
          '|'+color_table_code+'|'+fov_ratio_strg
    ENDIF ELSE $
       line=line+'|'+date+'|'+STRTRIM(sec,2) +'|'+outfn+'.zip'+$
          '|'+color_table_code

    line_exists=0
    IF file_exist(listfn) THEN BEGIN
      ;
      openr, listfnnr, listfn, /get_lun
      readline=''
      inputlistfn=''
      WHILE not EOF(listfnnr) DO BEGIN 
	readf, listfnnr, readline
	inputlistfn=[inputlistfn, readline]
      ENDWHILE
      close, listfnnr & free_lun, listfnnr
      ;
      ; check if current line exists already
      line_exists=max(line EQ inputlistfn[*])
      IF not line_exists THEN inputlistfn=[inputlistfn, line]
    ENDIF ELSE inputlistfn=['',line]
    IF not line_exists THEN BEGIN
	inputlistfn=inputlistfn[1:*]
	openw, listfnnr, listfn, /get_lun
	for i= 0, n_elements(inputlistfn)-1 DO $
           printf, listfnnr, inputlistfn[i]
        close, listfnnr & free_lun, listfnnr
    ENDIF

; sort list and remove double entries
;    spawn,'sort list.txt > list.txt2'
    spawn,'sort list.txt |uniq --check-chars=81 > list.txt2'
    spawn,'mv list.txt2 list.txt'


 ; say bye
   if keyword_set(verbose) then $
      printf, wzlog, '<<<< '+myname+'.pro finished at '+ systime()
      
   IF keyword_set(logfile) THEN BEGIN
      close, wzlog
      free_lun, wzlog
  ENDIF

; change back to where you come from

END
